import React, { useCallback, useEffect, useState } from "react";
import { View } from "react-native";
import { Formik } from "formik";
import { UploadAttachment } from "./molecules/UploadAttachment";
import * as Yup from "yup";
import { arrayLength, phoneNumPatternForLogin, removeArrays } from "../helpers";
import { IBusiness } from "../typings/business";
import { showMessage } from "react-native-flash-message";
import analytics from "../utils/segment";
import { CBFS_INITIAL_DATA } from "../data";
import { SEGMENT_EVENTS } from "../data/segmentEvents";
import { PLATFORM } from "../data/constant";
import { BankProps } from "../typings/bank";
import { useLMS } from "../context/context";
import { Text, useTheme } from "react-native-paper";
import { useAppErrors } from "../hooks";
import { Gutter } from "../component/Gutter";
import { FieldTextInput } from "../component/molecules/FieldTextInput";
import { FieldNumberInput } from "../component/molecules/FieldNumberInput";
import { FieldDropdownInput } from "../component/molecules/FieldDropdown";

const attachmentSchema = Yup.object({
  values: Yup.array().min(1, ({ path }) => {
    return `${path} is required`;
  }),
}).required();

const validationSchema = Yup.object().shape({
  fullName: Yup.string().required("Full Name is required"),
  email: Yup.string().email("valid email is required"),
  businessName: Yup.string().required("Business Name is required"),
  businessType: Yup.string().required(),
  businessAddress: Yup.string()
    .min(5, "Valid business address is required")
    .required("Business Address is required"),
  ntn: Yup.string()
    .min(7, "Valid NTN is required")
    .max(14, "Valid NTN is required")
    .required("NTN is required"),
  contactNumber: Yup.string()
    .matches(phoneNumPatternForLogin, "Phone number is not valid")
    .min(10, "Phone number should be 10 digits long.")
    .max(10, "Phone number should be 10 digits long."),
  partnerCnic: attachmentSchema.when("businessType", {
    is: "partnership",
    then: attachmentSchema.required(),
  }),
  partnershipDeed: attachmentSchema.when("businessType", {
    is: "partnership",
    then: attachmentSchema.required(),
  }),
  authorityLetter: attachmentSchema.when("businessType", {
    is: "partnership",
    then: attachmentSchema.required(),
  }),
  directorsResolution: attachmentSchema.when("businessType", {
    is: "company",
    then: attachmentSchema.required(),
  }),
  memorandumArticlesAssociation: attachmentSchema.when("businessType", {
    is: "company",
    then: attachmentSchema.required(),
  }),
  boardMemberCnic: attachmentSchema.when("businessType", {
    is: "company",
    then: attachmentSchema.required(),
  }),
  pocCnic: attachmentSchema.when("businessType", {
    is: "company",
    then: attachmentSchema.required(),
  }),
  beneficialOwnerCnic: attachmentSchema.when("businessType", {
    is: "company",
    then: attachmentSchema.required(),
  }),
  cnicFront: attachmentSchema.when("businessType", {
    is: "soleProprietorship" || "individual",
    then: attachmentSchema.required(),
  }),
  cnicBack: attachmentSchema.when("businessType", {
    is: "soleProprietorship" || "individual",
    then: attachmentSchema.required(),
  }),
  proprietorshipDeclaration: attachmentSchema.when("businessType", {
    is: "soleProprietorship",
    then: attachmentSchema.required(),
  }),
  ntnCertificate: attachmentSchema.when("businessType", {
    is: "soleProprietorship",
    then: attachmentSchema.required(),
  }),

  permissionLetter: attachmentSchema.when("businessType", {
    is: "branchOffice",
    then: attachmentSchema.required(),
  }),
  signatoryPassports: attachmentSchema.when("businessType", {
    is: "branchOffice",
    then: attachmentSchema.required(),
  }),
  formIIParticulars: attachmentSchema.when("businessType", {
    is: "branchOffice",
    then: attachmentSchema.required(),
  }),
  principalOfficeAuthorization: attachmentSchema.when("businessType", {
    is: "branchOffice",
    then: attachmentSchema.required(),
  }),

  bankDetails: Yup.object().shape({
    accountTitle: Yup.string().test("accountTitle", (num, values) => {
      const {
        parent: { accountNumber, accountTitle, iban },
      } = values;

      if (
        values.options.context &&
        values.options.context.isBankRequired &&
        !accountTitle
      ) {
        return values.createError({
          path: values.path,
          message: "Account Title is required",
        });
      }

      if (!!accountNumber || !!iban) {
        if (!accountTitle)
          return values.createError({
            path: values.path,
            message: "Account Title is required",
          });
      }
      return true;
    }),
    accountNumber: Yup.string().test("accountNumber", (num, values) => {
      const {
        parent: { accountNumber, accountTitle, iban },
      } = values;

      if (
        values.options.context &&
        values.options.context.isBankRequired &&
        !accountNumber
      ) {
        return values.createError({
          path: values.path,
          message: "Account Number is required",
        });
      }

      if (!!accountNumber || !!accountTitle || !!iban) {
        if (!accountNumber)
          return values.createError({
            path: values.path,
            message: "Account Number is required",
          });
      }
      return true;
    }),
    iban: Yup.string(),
  }),
});

export const RegisterFinance: React.ComponentType<{
  onSuccess?(business: IBusiness): void;
  isBankRequired?: boolean;
}> = ({ onSuccess, isBankRequired = false }) => {
  const [banks, setBanks] = useState<BankProps[]>([]);
  const [loading, setLoading] = useState(true);
  const { axiosInstance, setInfo, info, userDetail } = useLMS();
  const theme = useTheme();
  const { setAppError } = useAppErrors();

  const onSubmit = async (value: any, action: any) => {
    try {
      action.setSubmitting(true);
      // Convert into desire format
      const tempValue: any = Object.keys(value).reduce((acc, key) => {
        if (value[key]?.isAttachment) {
          return {
            ...acc,
            [key]: value[key].values,
          };
        } else {
          return {
            ...acc,
            [key]: value[key],
          };
        }
      }, {});

      if (
        tempValue.bankDetails.accountTitle === "" &&
        tempValue.bankDetails.accountNumber === "" &&
        tempValue.bankDetails.iban === ""
      ) {
        delete tempValue.bankDetails;
      }

      const finalValue = {
        ...tempValue,
        contactNumber: "+92" + tempValue.contactNumber,
      };
      const documentLength = arrayLength(finalValue);

      const { data }: { data: IBusiness } = await axiosInstance.post(
        "/v1/external-lending/sme/users",
        finalValue,
      );
      showMessage({
        message: "Successfully created",
        type: "success",
      });

      analytics?.identify(data.kyb.id);
      onSuccess?.(data);
      setInfo?.({ email: data.kyb.email });
      action.resetForm({
        values: CBFS_INITIAL_DATA,
      });

      analytics?.track(SEGMENT_EVENTS.COMPLETE_KYC_REGISTRATION, {
        ...removeArrays(finalValue),
        ...PLATFORM,
        attached_documents_count: documentLength,
      });
    } catch (e: any) {
      setAppError(e?.response?.data);
    } finally {
      action.setSubmitting(false);
    }
  };

  const fetchBank = useCallback(async () => {
    try {
      const { data } = await axiosInstance.get(
        "/v1/external-lending/sme/banks",
      );
      setBanks(data.banks);
    } catch (e) {
      showMessage({
        message: "Unable to fetch bank list",
        type: "danger",
      });
    }
  }, [axiosInstance]);

  useEffect(() => {
    setLoading(false);
  }, [info, userDetail]);

  useEffect(() => {
    fetchBank();
  }, [fetchBank]);

  console.log("bank", banks);

  return (
    <View>
      {!loading && (
        <Formik
          initialValues={{ ...CBFS_INITIAL_DATA, isBankRequired }}
          validationSchema={validationSchema}
          onSubmit={onSubmit}
        >
          {({
            values,
            handleSubmit,
            isSubmitting,
            setValues,
            setFieldValue,
            dirty,
            isValid,
          }) => (
            <View>
              <Text variant={"headlineSmall"} style={{ fontWeight: "500" }}>
                Register for Financing
              </Text>
              <Gutter spacing={0.2} />
              <Text variant={"bodyLarge"}>
                For helping us understand your business better, please provide
                the required detail
              </Text>
              <Gutter />
              <Text variant={"titleLarge"} style={{ fontWeight: "500" }}>
                Personal Details
              </Text>
              <Gutter spacing={0.2} />
              <FieldTextInput
                placeholder="Enter Full Name"
                name={"fullName"}
                title={"Full Name"}
                onBlur={() =>
                  analytics.track(SEGMENT_EVENTS.START_KYC_REGISTRATION, {
                    ...PLATFORM,
                  })
                }
              />
              <Gutter spacing={0.2} />
              <FieldNumberInput
                title="Contact Number"
                placeholder={"3001XXXXXX"}
                name={"contactNumber"}
                wrapperStyle={{ width: "100%" }}
                left={<Text variant={"bodySmall"}>+92</Text>}
              />
              <Gutter spacing={0.2} />
              <FieldTextInput
                placeholder="Enter email address"
                name={"email"}
                style={{
                  width: "100%",
                }}
                title="Email"
              />

              <Gutter spacing={1.5} />
              <Text variant={"titleLarge"} style={{ fontWeight: "500" }}>
                Bank Details
              </Text>
              <FieldDropdownInput
                name={"bank"}
                options={[{ label: "avinhs", value: "kumar" }]}
              />
              <Gutter spacing={0.5} />
              <FieldTextInput
                name="bankDetails.accountTitle"
                placeholder={"Account Title"}
                title={"Account Title"}
              />
              <Gutter spacing={0.5} />
              <FieldNumberInput
                placeholder="Account Number"
                name={"bankDetails.accountNumber"}
                title="Account Number"
              />
              <Gutter spacing={0.5} />
              <FieldTextInput
                placeholder="IBAN"
                name={"bankDetails.iban"}
                title="IBAN"
              />
              <Gutter spacing={1.5} />
              <Text variant={"titleLarge"} style={{ fontWeight: "500" }}>
                Business Details
              </Text>
              <FieldTextInput
                title={"Business Name"}
                name="businessName"
                placeholder={"Enter Business Name"}
              />
              <Gutter spacing={0.5} />
              {/*<UploadAttachment name={"Avinash"} title={"Avinash kumar"} />*/}
            </View>
          )}
        </Formik>
      )}
    </View>
  );
};

/*



									<Gutter spacing={1.5} />
									<Typography
										variant={'body1'}
										fontWeight={600}
									>
										Bank Details
									</Typography>
									<Col>
										<Label>Bank Name</Label>
										<Select
											labelId="demo-multiple-name-label"
											id="demo-multiple-name"
											value={values.bankDetails.bankCode}
											onChange={event =>
												setFieldValue(
													'bankDetails.bankCode',
													event.target.value
												)
											}
											input={
												<OutlinedInput
													size={'small'}
													sx={{
														width: '100%',
														borderRadius: '8px'
													}}
												/>
											}
											MenuProps={MenuProps}
										>
											{banks.map(bank => (
												<MenuItem
													key={bank.id}
													value={bank.bank_code}
													style={getStyles(theme)}
												>
													{bank.bank_name}
												</MenuItem>
											))}
										</Select>
									</Col>
									<Gutter spacing={0.5} />
									<FieldTextInput
										name="bankDetails.accountTitle"
										placeholder={'Account Title'}
										size="small"
										wrapperStyle={{ width: '100%' }}
										title={'Account Title'}
										style={{ width: '100%' }}
									/>
									<Gutter spacing={0.5} />
									<FieldNumberInput
										placeholder="Account Number"
										name={'bankDetails.accountNumber'}
										wrapperStyle={{ width: '100%' }}
										title="Account Number"
										style={{
											width: '100%'
										}}
										size="small"
									/>
									<Gutter spacing={0.5} />
									<FieldTextInput
										placeholder="IBAN"
										name={'bankDetails.iban'}
										wrapperStyle={{ width: '100%' }}
										title="IBAN"
										style={{
											width: '100%'
										}}
										size="small"
									/>
									<Gutter spacing={1.5} />
									<Typography
										variant={'body1'}
										fontWeight={600}
									>
										Business Details
									</Typography>
									<FieldTextInput
										title={'Business Name'}
										wrapperStyle={{ width: '100%' }}
										name="businessName"
										placeholder={'Enter Business Name'}
										size="small"
										style={{ width: '100%' }}
									/>
									<Gutter spacing={0.5} />
									<Col>
										<Label>Business Type</Label>
										<Select
											labelId="demo-multiple-name-label"
											id="demo-multiple-name"
											value={values.businessType}
											onChange={event => {
												setValues({
													...(removeKeysWithIsAttachment(
														values
													) as any),
													businessType:
														event.target.value,
													...BusinessType[
														event.target
															.value as KYBBusinessType
													]
												})
											}}
											input={
												<OutlinedInput
													size={'small'}
													sx={{
														width: '100%',
														borderRadius: '8px'
													}}
												/>
											}
											MenuProps={MenuProps}
										>
											{BUSINESS_TYPE.map(name => (
												<MenuItem
													key={name.name}
													value={name.value}
													style={getStyles(theme)}
												>
													{name.name}
												</MenuItem>
											))}
										</Select>
									</Col>
									<Gutter spacing={0.5} />
									<FieldNumberInput
										placeholder={`Enter ${values.businessType === 'soleProprietorship' || values.businessType === 'individual' ? 'CNIC' : 'NTN'} number`}
										name={'ntn'}
										wrapperStyle={{ width: '100%' }}
										style={{
											width: '100%'
										}}
										title={
											values.businessType ===
												'soleProprietorship' ||
											values.businessType === 'individual'
												? 'CNIC'
												: 'NTN'
										}
										size="small"
										inputProps={{ maxLength: 14 }}
									/>
									<Gutter spacing={0.5} />
									<FieldTextInput
										multiline
										title={'Business Address'}
										rows={2}
										wrapperStyle={{ width: '100%' }}
										name="businessAddress"
										placeholder={'Enter Business Address'}
										size="small"
										style={{ width: '100%' }}
									/>
									<Gutter spacing={1.5} />
								</Col>
								<Gutter spacing={1.5} />
								<Typography variant={'body1'} fontWeight={600}>
									Additional Details
								</Typography>
								<Gutter spacing={0.7} />
								{findKeysWithIsAttachment(values).map(m => {
									const attachment = values[
										m
									] as AttachementType

									return (
										<Col key={m}>
											<UploadAttachment
												name={`${m}.values`}
												title={`${attachment.title}${attachment.required ? '*' : ''}`}
											/>
										</Col>
									)
								})}
								<StyledRow>
									<StyledPrimaryButton
										variant={'contained'}
										onClick={() => handleSubmit()}
										disabled={
											isSubmitting || !isValid || !dirty
										}
									>
										<PrimaryButtonWrapper>
											Submit
										</PrimaryButtonWrapper>
									</StyledPrimaryButton>
								</StyledRow>
							</>
						)}
					</Formik>
				)}
			</Wrapper>
		</Card>
	)
}
*/
